#include "StdAfx.h"
#include "cTelnetClient.h"
#include "MainForm.h"

using namespace VaxTeleConfigSIP;

cTelnetClient::cTelnetClient(void)
{
	m_sAsyncBuffer = gcnew array<System::Byte>(512);
	m_objSocket = nullptr;
}

void cTelnetClient::OnReceive(IAsyncResult^ ar)
{
	cTelnetClient^ objTelnetClient = dynamic_cast<cTelnetClient^>(ar->AsyncState);
	
	Monitor::Enter(m_objLock);
		
	if (m_objSocket == nullptr)
	{
		Monitor::Exit(m_objLock);
		return;
	}
		
	try
	{
		int nRead = m_objSocket->EndReceive(ar);

		if (nRead == 0)
		{
			objTelnetClient->OnDisconnect();
			Monitor::Exit(m_objLock);
			return;
		}

		m_sRecvBuffer = m_sRecvBuffer + Encoding::ASCII->GetString(m_sAsyncBuffer, 0, nRead);
		m_sRecvBuffer = m_sRecvBuffer->Replace("VaxTele:>", "");

		objTelnetClient->ReceiveData();

		if (m_nAuthStatus != AUTH_STATUS_FAIL)
			m_objSocket->BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, ::SocketFlags::None, gcnew AsyncCallback(OnReceive), objTelnetClient);
	}
	catch (SocketException^ e)
	{
		objTelnetClient->OnDisconnect();
	}

	Monitor::Exit(m_objLock);
}

void cTelnetClient::ReceiveData()
{
	while (true)
	{
		if (m_nAuthStep != AUTH_DONE)
		{
			OnAuthrization(m_sRecvBuffer);
			break;
		}

        String^ sRequest = FindRequest();
		if (sRequest->Length == 0) break;

        String^ sCommand = gcnew String("");
        ArrayList^ objParamList = gcnew ArrayList();

        if (ParseRequest(sRequest, sCommand, objParamList) == false)
			break;
                                                
		OnRequest(sCommand, objParamList);
	}
}

void cTelnetClient::OnRequest(String^ sCommand, ArrayList^ objParamList)
{
	if (String::Compare(sCommand, "BYE", true) == 0)
    {
		OnDisconnect();
		return;
	}

	MainForm::m_objMainForm->InvokeTelnetRecv(sCommand, objParamList);
}

bool cTelnetClient::Connect(String^ sToIP, int nToPort, String^ sLogin, String^ sPassword)
{
	CloseSocket();
            
	m_objSocket = gcnew Socket(::AddressFamily::InterNetwork, ::SocketType::Stream, ::ProtocolType::Tcp);
            
    m_objSocket->BeginConnect(sToIP, nToPort, nullptr, nullptr);

	for (int nCount = 0; nCount < 30; nCount++)
	{
		Thread::Sleep(100);

		if (m_objSocket->Connected == true)
			break;
	}

	if (m_objSocket->Connected == false)
	{
		MessageBox::Show("Fail to connect to server.", "Socket", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return false;
	}

	m_sLoginTelnet = gcnew String(sLogin);
	m_sPasswordTelnet = gcnew String(sPassword);

    m_nAuthStep = AUTH_LOGIN;
    m_nAuthStatus = AUTH_STATUS_WAIT;
    m_sRecvBuffer = gcnew String("");

	m_objSocket->BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags::None, gcnew AsyncCallback(&cTelnetClient::OnReceive), this);

	for (int nCount = 0; nCount < 40; nCount++)
	{
		Thread::Sleep(100);

		if (m_nAuthStatus != AUTH_STATUS_WAIT)
			break;
	}

	if (m_nAuthStatus != AUTH_STATUS_SUCCESS)
    {
		MessageBox::Show("Fail to connect to server.", "Socket", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
		return false;
	}

	MessageBox::Show("Connected successfully.", "Socket", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);

	return true;
}

String^ cTelnetClient::FindRequest()
{
	String^ sCommand = gcnew String("");

	int nStart = m_sRecvBuffer->IndexOf("\r\n");
    if (nStart == -1) return sCommand;

    sCommand = m_sRecvBuffer->Substring(0, nStart);
    m_sRecvBuffer = m_sRecvBuffer->Remove(0, nStart + 2);

    return sCommand;
}

void cTelnetClient::OnAuthrization(String^ sData)
{
    if (m_nAuthStep == AUTH_LOGIN)
    {
        if (sData->IndexOf("Login:") == -1)
            return;

        m_nAuthStep = AUTH_PASSWORD;
        SendCommand(m_sLoginTelnet);

        m_sRecvBuffer = "";
    }
    else if (m_nAuthStep == AUTH_PASSWORD)
    {
        if (sData->IndexOf("Password:") == -1)
            return;

        m_nAuthStep = AUTH_WAIT;
        SendCommand(m_sPasswordTelnet);

        m_sRecvBuffer = "";
    }
    else if (m_nAuthStep == AUTH_WAIT && sData->Length != 0)
    {
        m_sRecvBuffer = "";

        if (sData->IndexOf("Success") == -1)
        {
            m_nAuthStep = AUTH_LOGIN;
            m_nAuthStatus = AUTH_STATUS_FAIL;

            return;
        }

        m_nAuthStep = AUTH_DONE;
        m_nAuthStatus = AUTH_STATUS_SUCCESS;
    }
}

bool cTelnetClient::ParseRequest(String^ sRequest, String^% sCommand, ArrayList^% objParamList)
{
    int nEnd = sRequest->IndexOf(" ");

    if(nEnd == -1)
    {
        sCommand = sRequest;
        return true;
    }

    sCommand = sRequest->Substring(0, nEnd);
    sRequest = sRequest->Remove(0, nEnd + 1);

	if(String::Compare(sCommand, "MSG", true) == 0 || String::Compare(sCommand, "StatusServerSIP", true) == 0 || String::Compare(sCommand, "ErrorLog", true) == 0 || String::Compare(sCommand, "StatusLog", true) == 0)
    {
        objParamList->Add(sRequest);
        return true;
    }

    while(true)
    {
        nEnd = sRequest->IndexOf(" ");

        if(nEnd == -1)
        {
            objParamList->Add(sRequest);
            break;
        }

        objParamList->Add(sRequest->Substring(0, nEnd));
        sRequest = sRequest->Remove(0, nEnd + 1);

    }

    return true;
}

void cTelnetClient::SendCommand(String^ sCommand, ArrayList^ objParamList)
{
    if (m_objSocket == nullptr)
        return;
    
    String^ sToData = gcnew String("");
	sToData = sCommand;

    for (int nCount = 0; nCount < objParamList->Count; nCount++)
        sToData = sToData + " " + objParamList[nCount];

    sToData += "\r\n";
	m_objSocket->Send(Encoding::UTF8->GetBytes(sToData), 0, sToData->Length, SocketFlags::None);
}

void cTelnetClient::SendCommand(String^ sCommand)
{
    if (m_objSocket == nullptr)
        return;
    
	String^ sToData = gcnew String("");
    
	sToData = sCommand + "\r\n";
	m_objSocket->Send(Encoding::UTF8->GetBytes(sToData), 0, sToData->Length, SocketFlags::None);
}

void cTelnetClient::OnDisconnect()
{
    CloseSocket();
	MessageBox::Show("Connection to host lost.", "Socket", MessageBoxButtons::OK, MessageBoxIcon::Exclamation);
}

void cTelnetClient::CloseSocket()
{
    if (m_objSocket != nullptr)
    {
        Monitor::Enter(m_objLock);
        
		m_objSocket->Close();
        m_objSocket = nullptr;
        
		Monitor::Exit(m_objLock);
    }

    m_nAuthStatus = AUTH_STATUS_FAIL;
}
