// ServerSIP.cpp: implementation of the CServerSIP class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ServerSIP.h"
#include "Registry.h"
#include "TelnetServer.h"
#include "TelnetClient.h"
#include "TelnetCmd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CServerSIP* CServerSIP::m_pServerSIP = NULL;

CServerSIP::CServerSIP()
{
	CServerSIP::m_pServerSIP = this;
	m_bStatusRun = FALSE;
}

CServerSIP::~CServerSIP()
{

}

BOOL CServerSIP::StartServerSIP()
{
	if(!m_bStatusRun)
		m_bStatusRun = InitVaxTeleSIP();

	CStringList objParamList;

	if(m_bStatusRun)
	{
		objParamList.AddTail("SIP Server is running.");
		CTelnetServer::m_pTelnetServer->SendCommandToAll("StatusServerSIP", &objParamList);
	}
	else
	{
		objParamList.AddTail("Unable to start SIP Server (see error log).");
		CTelnetServer::m_pTelnetServer->SendCommandToAll("StatusServerSIP", &objParamList);
	}

	return m_bStatusRun;
}

void CServerSIP::StopServerSIP()
{
	if(m_bStatusRun)
	{
		UnInitVaxTeleSIP();
		m_bStatusRun = FALSE;
	}

	CStringList objParamList;
	objParamList.AddTail("SIP Server is not running.");

	CTelnetServer::m_pTelnetServer->SendCommandToAll("StatusServerSIP", &objParamList);
}

void CServerSIP::UnInitVaxTeleSIP()
{
	LogStatus(" ");
	
	m_objCallList.DeleteAll();
	m_objServerData.UnLoadServer();
	
	LogStatus("Uninitializing VaxTele SIP Server.");
	UnInitialize();
	LogStatus("VaxTele SIP Server uninitialized successfully.");
	
	LogStatus("Unloading COM component.");
	UnInitObjectCOM();
	LogStatus("COM component unloaded successfully.");
	
	m_objUserList.DeleteAll();
	LogStatus("VaxTele SIP Server stopped successfully.");
}

BOOL CServerSIP::InitVaxTeleSIP()
{
	LogStatus(" ");
	LogStatus("Loading COM component.");

	if(!InitObjectCOM())
	{
		LogStatus("Unable to load COM component (see error log).");
		
		LogError("Unable to initialize the COM object.");
		LogError("Register the \"VaxTeleServerSIP.dll\" again using regsvr32 command.");
		
		return FALSE;
	}

	LogStatus("COM component loaded successfully.");
	
	CRegistry objReg;
	if(!objReg.GetStartServer()) return FALSE;
	
	CString sListenIP, sDomainRealm; CString sListenPort;
	objReg.GetCOMM(sListenIP, sListenPort, sDomainRealm);

	m_objClientRegEvent.SetDomainRealm(sDomainRealm);
		
	SetLicenseKey("TRIAL-LICENSE-KEY");

	LogStatus("Initializing VaxTele SIP Server.");
	
	if(!Initialize(sDomainRealm, sListenIP, atol(sListenPort), sListenIP, -1))
	{
		LogStatus("Unable to initialize VaxTele SIP Server (see error log).");
		LogError("");

		return FALSE;
	}

	LogStatus("VaxTele SIP Server initialized successfully.");
	
	m_objServerData.LoadServer();

	LogStatus("VaxTele SIP Server started successfully.");
						
	return TRUE;
}

void CServerSIP::BindTwoCall(CCallData* pCallDataA, CCallData* pCallDataB)
{
	pCallDataA->m_pBindCallData = pCallDataB;
	pCallDataB->m_pBindCallData = pCallDataA;
}

BOOL CServerSIP::SendResponse(CCallData* pCallData, int nStatusCode, LPCTSTR sReasonPhrase)
{
	if(pCallData->m_bConnected) return FALSE;

	return CVaxTeleCOM::SendResponse(pCallData->m_lCallId, nStatusCode, sReasonPhrase, "");
}

void CServerSIP::CloseCall(CCallData* pCallData)
{
	switch(pCallData->m_nCallType)
	{
		case CALL_TYPE_QUEUE:
			m_objQueueList.RemoveQueueCall(pCallData);
			break;
		
		case CALL_TYPE_STEALTH:
			m_objStealthList.RemoveStealthCall(pCallData);
			break;
	}

	PlayWaveStopToCall(pCallData);
	m_objStealthList.FreeCaptureCall(pCallData);
		
	CloseSession(pCallData);
	CVaxTeleCOM::CloseCall(pCallData->m_lCallId);
			
	CUserData* pUserData = m_objUserList.FindUser(pCallData->m_lUserId);
	if(pUserData != NULL) pUserData->m_lCallCount--;
	
	m_objCallList.RemoveCall(pCallData);
}

CCallData* CServerSIP::DialCallToLine(CCallData* pFromCallData, LPCTSTR sDialNo, LPCTSTR sLineName)
{
	CLineData* pLineData = m_objLineList.FindLine(sLineName);
	
	if(pLineData == NULL)
	{
		SendResponse(pFromCallData, 404, "User Not Found");
		CloseCall(pFromCallData);

		return NULL;
	}
	
	long lDialCall = CVaxTeleCOM::DialCallToLine(pLineData->m_lLineId, sDialNo, pLineData->m_sCodecList, 120);
	
	if(lDialCall == -1)
	{
		LONG lError = GetVaxObjectError();
		
		SendResponse(pFromCallData, 404, "User Not Found");
		CloseCall(pFromCallData);

		return NULL;
	}

	CCallData* pDialCallData = m_objCallList.AddCall(lDialCall);
		
	BindTwoCall(pFromCallData, pDialCallData);

	return pDialCallData;
}

CCallData* CServerSIP::DialCallToUser(CCallData* pFromCallData, LPCTSTR sToUserId)
{
	CUserData* pUserData = m_objUserList.FindUser(sToUserId);
	
	if(pUserData == NULL)
	{
		SendResponse(pFromCallData, 404, "User Not Found");
		CloseCall(pFromCallData);

		return NULL;
	}
	
	long lDialCall = CVaxTeleCOM::DialCallToUser(pFromCallData->m_sCallerId, sToUserId, pUserData->m_sCodecList, 120);
	
	if(lDialCall == -1)
	{
		SendResponse(pFromCallData, 404, "User Not Found");
		CloseCall(pFromCallData);

		return NULL;
	}

	pUserData->m_lCallCount++;
		
	CCallData* pDialCallData = m_objCallList.AddCall(lDialCall);
	
	pDialCallData->m_sUserId = sToUserId;
	pDialCallData->m_lUserId = pUserData->m_lUserId;
		
	BindTwoCall(pFromCallData, pDialCallData);

	return pDialCallData;
}


BOOL CServerSIP::AcceptCall(CCallData* pCallData)
{
	if(!CVaxTeleCOM::AcceptCall(pCallData->m_lCallId, pCallData->m_sCodec, 120))
	{
		LogError("");
		LogStatus("CServerSIP.AcceptCall() method fails.");

		return FALSE;
	}

	return TRUE;
}

/*
long CServerSIP::CreateSession()
{
	return CVaxTeleCOM::CreateSession();
}
*/

void CServerSIP::CreateSession(CCallData* pCallData)
{
	CloseSession(pCallData);
	
	pCallData->m_lSessionId = CVaxTeleCOM::CreateSession();

	CVaxTeleCOM::AddCallToSession(pCallData->m_lCallId, pCallData->m_lSessionId);
}

void CServerSIP::AddCallToSession(CCallData* pCallData, long lSessionId)
{
	CVaxTeleCOM::AddCallToSession(pCallData->m_lCallId, lSessionId);
}

void CServerSIP::CloseSession(CCallData* pCallData)
{
	if(pCallData->m_lSessionId == -1) return;
	
	PlayWaveStopToCall(pCallData);
		
	CVaxTeleCOM::CloseSession(pCallData->m_lSessionId);
	pCallData->m_lSessionId = -1;
}

/*
void CServerSIP::CloseSession(long lSessionId)
{
	CVaxTeleCOM::CloseSession(lSessionId);
}
*/

void CServerSIP::TransferCallAccept(CCallData* pCallData)
{
	CVaxTeleCOM::TransferCallAccept(pCallData->m_lCallId);
}

void CServerSIP::DisableDetectDTMF(CCallData* pCallData)
{
	pCallData->m_bDigitDetect = FALSE;
	pCallData->m_nDigitPRESS  = 0;
	pCallData->m_sDigitPRESS  = "";

	CVaxTeleCOM::DetectDTMF(0, pCallData->m_lSessionId, FALSE); // Disable RFC2833 DTMF detection
}

void CServerSIP::EnableDetectDTMF(CCallData* pCallData)
{
	pCallData->m_bDigitDetect = TRUE;
	pCallData->m_nDigitPRESS  = 0;
	pCallData->m_sDigitPRESS  = "";

	if(!CVaxTeleCOM::DetectDTMF(0, pCallData->m_lSessionId, TRUE))  //Enable RFC2833 DTMF detection
		LogError("");
}

void CServerSIP::OnClientUnRegister(LPCTSTR sUserLogin) 
{
	m_objClientRegEvent.OnClientUnRegister(sUserLogin);
}

void CServerSIP::OnClientRegister(LPCTSTR sUserLogin, LPCTSTR sDomain, long lRegId) 
{
	m_objClientRegEvent.OnClientRegister(sUserLogin, sDomain, lRegId); 
}

void CServerSIP::OnClientRegisterSuccess(LPCTSTR sUserLogin) 
{
	m_objClientRegEvent.OnClientRegisterSuccess(sUserLogin);
}

void CServerSIP::OnClientRegisterFail(LPCTSTR sUserLogin) 
{
	m_objClientRegEvent.OnClientRegisterFail(sUserLogin);
}

void CServerSIP::OnIncomingCallAnonymous(long lCallId, LPCTSTR sCallFrom, LPCTSTR sCallTo, LPCTSTR sFromURI, LPCTSTR sToURI, LPCTSTR sFromIP, long lFromPort) 
{
	CVaxTeleCOM::SendResponse(lCallId, 404, "Anonymous not allowed.", "");
}

void CServerSIP::OnIncomingCallFromLine(long lCallId, long lLineId, LPCTSTR sCallFrom, LPCTSTR sCallTo, LPCTSTR sFromURI, LPCTSTR sToURI, LPCTSTR sFromIP, long lFromPort) 
{
	CLineData* pLineData = m_objLineList.FindLine(lLineId);
	if(pLineData == NULL) 
	{
		CVaxTeleCOM::CloseCall(lCallId);
		return;
	}
		
	CCallData* pCallData   = m_objCallList.AddCall(lCallId);
	
	pCallData->m_sCodec    = pLineData->m_sCodecList;
	pCallData->m_sCallerId = sCallFrom;

	CreateSession(pCallData);
		
	m_objCallEvent.OnIncomingCallFromLine(pCallData, pLineData, sCallTo);
}

void CServerSIP::OnIncomingCallFromUser(long lCallId, LPCTSTR sUserName, LPCTSTR sCallFrom, LPCTSTR sCallTo, LPCTSTR sFromURI, LPCTSTR sToURI, LPCTSTR sFromIP, long lFromPort) 
{
	CUserData* pUserData = m_objUserList.FindUser(sUserName);
	if(pUserData == NULL) return;
		
	CCallData* pCallData = m_objCallList.AddCall(lCallId);
	
	pCallData->m_sCodec    = pUserData->m_sCodecList;
	pCallData->m_lUserId   = pUserData->m_lUserId;
	pCallData->m_sUserId   = sUserName;
	pCallData->m_sCallerId = sCallFrom;
	
	CreateSession(pCallData);

	EnableDialTone(lCallId, m_objServerData.GetDialToneSessionId());

	m_objCallEvent.OnIncomingCallFromUser(pCallData, pUserData, sCallTo);
}

void CServerSIP::StartVoiceStream(CCallData* pCallDataA, CCallData* pCallDataB)
{
	CloseSession(pCallDataA);
	CloseSession(pCallDataB);

	long lSessionId = CVaxTeleCOM::CreateSession();

	pCallDataA->m_lSessionId = lSessionId;
	pCallDataB->m_lSessionId = lSessionId;

	AddCallToSession(pCallDataA, lSessionId);
	AddCallToSession(pCallDataB, lSessionId);
}

void CServerSIP::OnAcceptIncomingCallConnected(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;

	m_objCallEvent.OnAcceptIncomingCallConnected(pCallData);
}

void CServerSIP::OnDialCallConnected(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objCallEvent.OnDialCallConnected(pCallData);
}

void CServerSIP::OnDialCallTimeOut(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;

	m_objCallEvent.OnDialCallTimeOut(pCallData);
}

void CServerSIP::OnAcceptCallTimeOut(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;

	m_objCallEvent.OnAcceptCallTimeOut(pCallData);
}

void CServerSIP::OnDisconnectCall(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objCallEvent.OnDisconnectCall(pCallData);
}

void CServerSIP::OnIncomingCallCancel(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objCallEvent.OnIncomingCallCancel(pCallData);
}

void CServerSIP::OnSessionLost(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objCallEvent.OnSessionLost(pCallData);
}

void CServerSIP::OnProvisionalResponse(long lCallId, long lStatusCode, LPCTSTR sReasonPhrase) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;

	m_objResponseEvent.OnProvisionalResponse(pCallData, lStatusCode, sReasonPhrase);
}

void CServerSIP::OnFailureResponse(long lCallId, long lStatusCode, LPCTSTR sReasonPhrase) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objResponseEvent.OnFailureResponse(pCallData, lStatusCode, sReasonPhrase); 
}

void CServerSIP::OnRedirectionResponse(long lCallId, long lStatusCode, LPCTSTR sReasonPhrase, LPCTSTR sContact) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objResponseEvent.OnRedirectionResponse(pCallData, lStatusCode, sReasonPhrase, sContact);
}

void CServerSIP::OnDigitDetectDTMF(long lSessionId, long lCallId, long lDigit) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;

	char sDigit[20] = "0123456789*#ABCDEF";

	m_objQueueList.OnPressDigitDTMF(pCallData, sDigit[lDigit]);
}

void CServerSIP::OnHoldCall(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objHoldCallEvent.OnHoldCall(pCallData);
}

void CServerSIP::OffHoldCall(long lCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objHoldCallEvent.OffHoldCall(pCallData);
}

void CServerSIP::OnTransferCallBlind(long lCallId, LPCTSTR sTransferee, LPCTSTR sTransferTo, LPCTSTR sTransferToURI) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;
	
	m_objTransferCallEvent.OnTransferCallBlind(pCallData, sTransferee, sTransferTo, sTransferToURI);
}

void CServerSIP::OnTransferCallAttend(long lCallId, long lTransferToCallId) 
{
	CCallData* pCallData = m_objCallList.FindCall(lCallId);
	if(pCallData == NULL) return;

	CCallData* pTransferToCallData = m_objCallList.FindCall(lTransferToCallId);
	if(pTransferToCallData == NULL) return;

	m_objTransferCallEvent.OnTransferCallAttend(pCallData, pTransferToCallData);	
}

BOOL CServerSIP::PlayWaveStopToCall(CCallData* pCallData)
{
	if(pCallData->m_nPlayWaveType == PLAY_WAVE_NONE_TYPE)
		return TRUE;
	
	if(pCallData->m_nPlayWaveType == PLAY_WAVE_DONE_TYPE)
		m_objPlayWaveList.RemoveKey(pCallData->m_lSessionId);
	
	pCallData->m_nPlayWaveType = PLAY_WAVE_NONE_TYPE;
	
	return CVaxTeleCOM::PlayWaveStopToSession(pCallData->m_lSessionId);
}

BOOL CServerSIP::PlayWaveStartToCall(CCallData* pCallData, long lPlayWaveId, BOOL bLoopPlay)
{
	if(bLoopPlay == FALSE)
	{
		pCallData->m_nPlayWaveType = PLAY_WAVE_DONE_TYPE;
		m_objPlayWaveList.SetAt(pCallData->m_lSessionId, pCallData);
	}
	else
	{
		pCallData->m_nPlayWaveType = PLAY_WAVE_LOOP_TYPE;
	}
	
	return CVaxTeleCOM::PlayWaveStartToSession(lPlayWaveId, pCallData->m_lSessionId, bLoopPlay);	
}

void CServerSIP::OnPlayWaveToSessionDone(long lPlayWaveId, long lSessionId) 
{
	CCallData* pCallData;
	if(!m_objPlayWaveList.Lookup(lSessionId, pCallData)) return;
	
	m_objPlayWaveList.RemoveKey(lSessionId);
	pCallData->m_nPlayWaveType = PLAY_WAVE_NONE_TYPE;

	m_objQueueList.OnPlayWaveDoneToCall(pCallData);
}

void CServerSIP::OnLineRegisterTrying(long lLineId) 
{
	m_objLineRegisterEvent.OnLineRegisterTrying(lLineId);
}

void CServerSIP::OnLineRegisterFail(long lLineId, long lStatusCode, LPCTSTR sReasonPhrase) 
{
	m_objLineRegisterEvent.OnLineRegisterFail(lLineId, lStatusCode, sReasonPhrase);
}

void CServerSIP::OnLineRegisterSuccess(long lLineId) 
{
	m_objLineRegisterEvent.OnLineRegisterSuccess(lLineId);
}

void CServerSIP::OnLineUnRegisterTrying(long lLineId) 
{
	m_objLineRegisterEvent.OnLineUnRegisterTrying(lLineId);
}

void CServerSIP::OnLineUnRegisterFail(long lLineId, long lStatusCode, LPCTSTR sReasonPhrase) 
{
	m_objLineRegisterEvent.OnLineUnRegisterFail(lLineId, lStatusCode, sReasonPhrase);
}

void CServerSIP::OnLineUnRegisterSuccess(long lLineId) 
{
	m_objLineRegisterEvent.OnLineUnRegisterSuccess(lLineId);
}

void CServerSIP::OnVaxTeleTick(long lTickId)
{
	if(lTickId == QUEUE_TICK_ID)
		m_objQueueList.ProcessQueueTick();
}

void CServerSIP::LogError(LPCTSTR sError)
{
	m_objErrorLog.LogError(sError);
}

void CServerSIP::LogStatus(LPCTSTR sStatus)
{
	CStringList objParamList;
	objParamList.AddTail(sStatus);

	CTelnetServer::m_pTelnetServer->SendCommandToAll("StatusLog", &objParamList);	
}

BOOL CServerSIP::LoadWaveFileToSession(LPCTSTR sFileName, long& lPlayWaveId, long& lSessionId)
{
	LogStatus("Loading [ " + CString(sFileName) + " ] wave file....");
	
	LONG lPlayWaveLoad = PlayWaveLoadFile(sFileName);
	
	if(lPlayWaveLoad == -1)
	{
		LogError("");
		LogStatus("Loading [ " + CString(sFileName) + " ] wave file fail.");
		return FALSE;
	}
	
	LONG lSessionPlayWave = CVaxTeleCOM::CreateSession();

	if(lSessionPlayWave == -1)
	{
		LogError("");
		LogStatus("Loading [ " + CString(sFileName) + " ] wave file fail.");

		PlayWaveUnLoadFile(lPlayWaveLoad);

		return FALSE;
	}
	
	if(!PlayWaveStartToSession(lPlayWaveLoad, lSessionPlayWave, TRUE))
	{
		LogError("");
		LogStatus("Loading [ " + CString(sFileName) + " ] wave file fail.");
		
		CVaxTeleCOM::CloseSession(lSessionPlayWave);
		CVaxTeleCOM::PlayWaveUnLoadFile(lPlayWaveLoad);

		return FALSE;
	}
	

	lPlayWaveId = lPlayWaveLoad;
	lSessionId  = lSessionPlayWave;

	LogStatus("Loading [ " + CString(sFileName) + " ] wave file done.");
	
	return TRUE;
}

void CServerSIP::UnLoadWaveFileToSession(long lPlayWaveId, long lSessionId)
{
	CVaxTeleCOM::CloseSession(lSessionId);
	CVaxTeleCOM::PlayWaveUnLoadFile(lPlayWaveId);
}

void CServerSIP::OnTelnetCommand(LPCTSTR sCommand, CStringList* pParamList, CTelnetClient* pTelnetClient)
{
	CTelnetCmd objTelnetCmd;

	if(::lstrcmpi(sCommand, "ShowUsers") == 0)
	{
		objTelnetCmd.ShowUsers(pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "UserInfo") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.UserInfo(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "AddUser") == 0 && pParamList->GetCount() == 5)
	{
		objTelnetCmd.AddUser(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "RemoveUser") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.RemoveUser(pParamList, pTelnetClient);
		return;
	}


	if(::lstrcmpi(sCommand, "ShowQueues") == 0)
	{
		objTelnetCmd.ShowQueues(pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "QueueInfo") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.QueueInfo(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "AddQueue") == 0 && pParamList->GetCount() == 6)
	{
		objTelnetCmd.AddQueue(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "RemoveQueue") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.RemoveQueue(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "ShowLines") == 0)
	{
		objTelnetCmd.ShowLines(pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "LineInfo") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.LineInfo(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "AddLine") == 0 && pParamList->GetCount() == 12)
	{
		objTelnetCmd.AddLine(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "RemoveLine") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.RemoveLine(pParamList, pTelnetClient);
		return;
	}

	///////////////////////////

	if(::lstrcmpi(sCommand, "ShowDialPlans") == 0)
	{
		objTelnetCmd.ShowDialPlans(pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "DialPlanInfo") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.DialPlanInfo(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "AddDialPlan") == 0 && pParamList->GetCount() == 3)
	{
		objTelnetCmd.AddDialPlan(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "RemoveDialPlan") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.RemoveDialPlan(pParamList, pTelnetClient);
		return;
	}

	///////////////////////////

	if(::lstrcmpi(sCommand, "SetStealthMusic") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.SetStealthMusic(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "GetStealthMusic") == 0)
	{
		objTelnetCmd.GetStealthMusic(pTelnetClient);
		return;
	}

	//////////////////////////

	if(::lstrcmpi(sCommand, "SetHoldMusic") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.SetHoldMusic(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "GetHoldMusic") == 0)
	{
		objTelnetCmd.GetHoldMusic(pTelnetClient);
		return;
	}

	//////////////////////////

	if(::lstrcmpi(sCommand, "SetDialToneMusic") == 0 && pParamList->GetCount() == 1)
	{
		objTelnetCmd.SetDialToneMusic(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "GetDialToneMusic") == 0)
	{
		objTelnetCmd.GetDialToneMusic(pTelnetClient);
		return;
	}

	//////////////////////////

	if(::lstrcmpi(sCommand, "SetCOMM") == 0 && pParamList->GetCount() == 3)
	{
		objTelnetCmd.SetCOMM(pParamList, pTelnetClient);
		return;
	}

	if(::lstrcmpi(sCommand, "GetCOMM") == 0)
	{
		objTelnetCmd.GetCOMM(pTelnetClient);
		return;
	}

	//////////////////////////

	if(::lstrcmpi(sCommand, "ServerStart") == 0)
	{
		objTelnetCmd.StartServer(pTelnetClient);
		
		StartServerSIP();
		return;
	}

	if(::lstrcmpi(sCommand, "ServerStop") == 0)
	{
		objTelnetCmd.StopServer(pTelnetClient);

		StopServerSIP();
		return;
	}

	if(::lstrcmpi(sCommand, "StatusServerSIP") == 0)
	{
		CStringList objParamList;
		
		if(m_bStatusRun)
		{
			objParamList.AddTail("SIP Server is running.");
			pTelnetClient->SendCommand("StatusServerSIP", &objParamList);
		}
		else
		{
			objParamList.AddTail("SIP Server is not running.");
			pTelnetClient->SendCommand("StatusServerSIP", &objParamList);
		}
	}

	////////////////////////

	if(::lstrcmpi(sCommand, "GetCallCount") == 0)
	{
		char sCallCount[16] = "\0";
		::sprintf(sCallCount, "%d", m_objCallList.GetCount());

		CStringList objParamList;
		objParamList.AddTail(sCallCount);

		pTelnetClient->SendCommand("GetCallCount", &objParamList);
	}

	if(::lstrcmpi(sCommand, "GetUserCount") == 0)
	{
		char sUserCount[16] = "\0";
		::sprintf(sUserCount, "%d", m_objUserList.GetCount());

		CStringList objParamList;
		objParamList.AddTail(sUserCount);

		pTelnetClient->SendCommand("GetUserCount", &objParamList);
	}

	if(::lstrcmpi(sCommand, "GetUsageCPU") == 0)
	{
		char sUsageCPU[16] = "\0";
		::sprintf(sUsageCPU, "%d", m_objCPU.GetUsageCPU());

		CStringList objParamList;
		objParamList.AddTail(sUsageCPU);

		pTelnetClient->SendCommand("GetUsageCPU", &objParamList);
	}

	
	if(::lstrcmpi(sCommand, "GetAvailMEM") == 0)
	{
		MEMORYSTATUS stat;
		GlobalMemoryStatus (&stat);
		
		char sAvailMEM[16] = "\0";
		::sprintf(sAvailMEM, "%d", stat.dwAvailPhys/1024);

		CStringList objParamList;
		objParamList.AddTail(sAvailMEM);

		pTelnetClient->SendCommand("GetAvailMEM", &objParamList);
	}


	////////////////////////

	objTelnetCmd.InvalidCommand(pTelnetClient);
}

