// ServerSIP.h: interface for the CServerSIP class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SERVERSIP_H__11D36B28_21F5_4BEE_B58C_E2787557CA3F__INCLUDED_)
#define AFX_SERVERSIP_H__11D36B28_21F5_4BEE_B58C_E2787557CA3F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "VaxTeleCOM.h"

#define CALL_TYPE_REGULAR	0
#define CALL_TYPE_QUEUE		1
#define CALL_TYPE_STEALTH	2

#define PLAY_WAVE_NONE_TYPE		100
#define PLAY_WAVE_DONE_TYPE		101
#define PLAY_WAVE_LOOP_TYPE		102	

#define QUEUE_TICK_ID				2001

#include "ServerSIP.h"
#include "CallList.h"
#include "UserList.h"
#include "QueueList.h"
#include "DialPlanList.h"
#include "LineList.h"
#include "StealthList.h"
#include "ErrorLog.h"
#include "CallEvent.h"
#include "ClientRegisterEvent.h"
#include "LineRegisterEvent.h"
#include "HoldCallEvent.h"
#include "TransferCallEvent.h"
#include "ResponseEvent.h"
#include "ServerData.h"
#include "TelnetClient.h"
#include "cpu.h"


class CServerSIP : public CVaxTeleCOM  
{

public:
	
	CServerSIP();
	~CServerSIP();

public:

	CPU m_objCPU;

	static CServerSIP* m_pServerSIP;

	CServerData m_objServerData;

	CTransferCallEvent   m_objTransferCallEvent;
	CHoldCallEvent		 m_objHoldCallEvent;
	CCallEvent           m_objCallEvent;
	CClientRegisterEvent m_objClientRegEvent;
	CLineRegisterEvent   m_objLineRegisterEvent;
	CResponseEvent		 m_objResponseEvent;		
		
	CErrorLog            m_objErrorLog;
	
	CCallList      m_objCallList;
	CQueueList     m_objQueueList;
	CDialPlanList  m_objDialPlanList;
	CLineList      m_objLineList;
	CUserList      m_objUserList;
	CStealthList   m_objStealthList;

	CMap <long, long, CCallData*, CCallData*> m_objPlayWaveList;
	
public:

	BOOL StartServerSIP();
	void StopServerSIP();

private:

	BOOL m_bStatusRun;
		
public:
	
	void BindTwoCall(CCallData* pCallDataA, CCallData* pCallDataB);

	void LogError(LPCTSTR sError);
	void LogStatus(LPCTSTR sStatus);
	
	BOOL InitVaxTeleSIP();
	void UnInitVaxTeleSIP();

	BOOL AcceptCall(CCallData* pCallData);

	CCallData* DialCallToLine(CCallData* pFromCallData, LPCTSTR sDialNo, LPCTSTR sLineName);
	CCallData* DialCallToUser(CCallData* pFromCallData, LPCTSTR sToUserId);
	
	void CloseCall(CCallData* pCallData);

	BOOL PlayWaveStartToCall(CCallData* pCallData, long lPlayWaveId, BOOL bLoopPlay);
	BOOL PlayWaveStopToCall(CCallData* pCallData);

	BOOL SendResponse(CCallData* pCallData, int nStatusCode, LPCTSTR sReasonPhrase);

	void DisableDetectDTMF(CCallData* pCallData);
	void EnableDetectDTMF(CCallData* pCallData);
	
	void CreateSession(CCallData* pCallData);
	void AddCallToSession(CCallData* pCallData, long lSessionId);
	void CloseSession(CCallData* pCallData);
	
	void StartVoiceStream(CCallData* pCallDataA, CCallData* pCallDataB);

	BOOL LoadWaveFileToSession(LPCTSTR sFileName, long& lPlayWaveId, long& lSessionId);
	void UnLoadWaveFileToSession(long lPlayWaveId, long lSessionId);

	void TransferCallAccept(CCallData* pCallData);
		
public:

	void OnTelnetCommand(LPCTSTR sCommand, CStringList* pParamList, CTelnetClient* pTelnetClient);

	void OnClientUnRegister(LPCTSTR sUserLogin);
	void OnClientRegister(LPCTSTR sUserLogin, LPCTSTR sDomain, long lRegId);
	void OnClientRegisterSuccess(LPCTSTR sUserLogin);
	void OnClientRegisterFail(LPCTSTR sUserLogin);

	void OnLineRegisterTrying(long lLineId);
	void OnLineRegisterFail(long lLineId, long lStatusCode, LPCTSTR sReasonPhrase);
	void OnLineRegisterSuccess(long lLineId);

	void OnLineUnRegisterTrying(long lLineId);
	void OnLineUnRegisterFail(long lLineId, long lStatusCode, LPCTSTR sReasonPhrase);
	void OnLineUnRegisterSuccess(long lLineId);

	void OnIncomingCallFromLine(long lCallId, long lLineId, LPCTSTR sCallFrom, LPCTSTR sCallTo, LPCTSTR sFromURI, LPCTSTR sToURI, LPCTSTR sFromIP, long lFromPort);
	void OnIncomingCallFromUser(long lCallId, LPCTSTR sUserName, LPCTSTR sCallFrom, LPCTSTR sCallTo, LPCTSTR sFromURI, LPCTSTR sToURI, LPCTSTR sFromIP, long lFromPort);
	void OnIncomingCallAnonymous(long lCallId, LPCTSTR sCallFrom, LPCTSTR sCallTo, LPCTSTR sFromURI, LPCTSTR sToURI, LPCTSTR sFromIP, long lFromPort);
	
	void OnIncomingCallCancel(long lCallId);
	void OnDialCallConnected(long lCallId);
	void OnAcceptIncomingCallConnected(long lCallId);
	void OnDisconnectCall(long lCallId);
	
	void OnDialCallTimeOut(long lCallId);
	void OnAcceptCallTimeOut(long lCallId);

	void OnProvisionalResponse(long lCallId, long lStatusCode, LPCTSTR sReasonPhrase);
	void OnFailureResponse(long lCallId, long lStatusCode, LPCTSTR sReasonPhrase);
	void OnRedirectionResponse(long lCallId, long lStatusCode, LPCTSTR sReasonPhrase, LPCTSTR sContact);

	void OnTransferCallBlind(long lCallId, LPCTSTR sTransferee, LPCTSTR sTransferTo, LPCTSTR sTransferToURI);
	void OnTransferCallAttend(long lCallId, long lTransferToCallId);

	void OnHoldCall(long lCallId);
	void OffHoldCall(long lCallId);
	
	void OnPlayWaveToSessionDone(long lPlayWaveId, long lSessionId);
	void OnDigitDetectDTMF(long lSessionId, long lCallId, long lDigit);
	void OnSessionLost(long lCallId);
	
	void OnVaxTeleTick(long lTickId);

};

#endif // !defined(AFX_SERVERSIP_H__11D36B28_21F5_4BEE_B58C_E2787557CA3F__INCLUDED_)
