// UserList.cpp: implementation of the CUserList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ServerSIP.h"
#include "UserList.h"
#include "Registry.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUserList::CUserList()
{
	m_pUserListTEXT = new CMapUserStrToPtr;
	m_pUserListINT  = new CMapUserULongToPtr;

	m_lUserIdCount = 0;

	m_pUserListTEXT->InitHashTable(256, TRUE);
	m_pUserListINT->InitHashTable(256, TRUE);
}

CUserList::~CUserList()
{
	DeleteAll();

	delete m_pUserListTEXT;
	delete m_pUserListINT;
}

ULONG CUserList::ChooseUserId()
{
	while(TRUE)
	{
		m_lUserIdCount++;

		if(FindUser(m_lUserIdCount) == NULL) break;
	}

	return m_lUserIdCount;
}

BOOL CUserList::AddUser(LPCTSTR sUserId, LPCTSTR sUserPwd, BOOL bStealthListener, LPCTSTR sCodecList, BOOL bBlock)
{
	if(!CServerSIP::m_pServerSIP->AddUser(sUserId, sUserPwd))
	{
		CServerSIP::m_pServerSIP->LogError("");
		return FALSE;
	}
	
	Remove(sUserId);

	Add(sUserId, bStealthListener, sCodecList, bBlock);

	return TRUE;
}

void CUserList::RemoveUser(CString sUserId)
{
	CServerSIP::m_pServerSIP->RemoveUser(sUserId);
	
	Remove(sUserId);
}

CUserData* CUserList::FindUser(ULONG lUserId)
{
	CUserData* pUserData;
	if(!m_pUserListINT->Lookup(lUserId, pUserData))
		return NULL;
	
	return pUserData;
}

CUserData* CUserList::FindUser(CString sUserId)
{
	sUserId.MakeLower();

	CUserData* pUserData;
	if(!m_pUserListTEXT->Lookup(sUserId, pUserData))
		return NULL;
	
	return pUserData;
}


void CUserList::DeleteAll()
{
	ULONG lKey;
	CUserData* pUserData;

	for(POSITION pos = m_pUserListINT->GetStartPosition(); pos != NULL;)
	{
		m_pUserListINT->GetNextAssoc( pos, lKey, pUserData);
		delete pUserData;
	}

	m_pUserListINT->RemoveAll();
	m_pUserListTEXT->RemoveAll();
}

int CUserList::GetCallCount(CString sUserId)
{
	CUserData* pUserData = FindUser(sUserId);
	if(pUserData == NULL) return -1;

	return pUserData->m_lCallCount;
}

void CUserList::Add(CString sUserId, BOOL bStealthListener, LPCTSTR sCodecList, BOOL bBlock)
{
	CUserData* pUserData = new CUserData;
	
	pUserData->m_sUserId          = sUserId;
	pUserData->m_lUserId          = ChooseUserId();
	pUserData->m_bStealthListener = bStealthListener;
	pUserData->m_sCodecList       = sCodecList;
	pUserData->m_bBlock			  = bBlock;
	
	m_pUserListINT->SetAt(pUserData->m_lUserId, pUserData);
	
	sUserId.MakeLower();
	m_pUserListTEXT->SetAt(sUserId, pUserData);
}

void CUserList::Remove(CString sUserId)
{
	CUserData* pUserData;
	sUserId.MakeLower();
	
	if(!m_pUserListTEXT->Lookup(sUserId, pUserData))
		return;
			
	m_pUserListINT->RemoveKey(pUserData->m_lUserId);
	m_pUserListTEXT->RemoveKey(sUserId);

	delete pUserData;
}

DWORD CUserList::GetCount()
{
	return m_pUserListINT->GetCount();
}