using System;
using System.Collections;
using System.Text;
using System.Diagnostics;

class cServerSIP
{
    private PerformanceCounter m_objUsageCPU = new PerformanceCounter("Processor", "% Processor Time", "_Total");

    public const int CALL_TYPE_REGULAR = 0;
    public const int CALL_TYPE_QUEUE = 1;
    public const int CALL_TYPE_STEALTH = 2;

    public const int PLAY_WAVE_NONE_TYPE = 100;
    public const int PLAY_WAVE_DONE_TYPE = 101;
    public const int PLAY_WAVE_LOOP_TYPE = 102;

    public const int QUEUE_TICK_ID = 2001;

    public const int QUEUE_CALL_TYPE = 100;
    public const int STEALTH_CALL_TYPE = 101;
    public const int BLIND_TRANSFER_CALL_TYPE = 102;

    public static cServerSIP m_objServerSIP;
    private cVaxTeleCOM m_objVaxTeleCOM = new cVaxTeleCOM();

    public cCallList m_objCallList = new cCallList();
    public cLineList m_objLineList = new cLineList();
    public cDialPlanList m_objDialPlanList = new cDialPlanList();
    public cQueueList m_objQueueList = new cQueueList();
    public cUserList m_objUserList = new cUserList();
    public cStealthList m_objStealthList = new cStealthList();

    public cServerData m_objServerData = new cServerData();
    public cErrorLog m_objErrorLog = new cErrorLog();

    private cClientRegisterEvent m_objClientRegEvent = new cClientRegisterEvent();
    private cCallEvent m_objCallEvent = new cCallEvent();
    private cResponseEvent m_objResponseEvent = new cResponseEvent();
    private cHoldCallEvent m_objHoldCallEvent = new cHoldCallEvent();
    private cTransferCallEvent m_objTransferCallEvent = new cTransferCallEvent();
    private cLineRegisterEvent m_objLineRegisterEvent = new cLineRegisterEvent();

    public Hashtable m_objPlayWaveList = new Hashtable();

    private Boolean m_bStatusRun;
          

  public cServerSIP()
  {
      m_objServerSIP = this;
      m_bStatusRun = false;
  }


  public Boolean StartServerSIP()
  {
        if(m_bStatusRun == false)
        {
            m_bStatusRun = InitVaxTeleSIP();
        }

        ArrayList objParamList = new ArrayList();

        if(m_bStatusRun == true)
        {
            objParamList.Add("SIP Server is running.");
            cTelnetServer.m_objTelnetServer.SendCommandToAll("StatusServerSIP", ref objParamList);
        }
        else
        {
            objParamList.Add("Unable to start SIP Server (see error log).");
            cTelnetServer.m_objTelnetServer.SendCommandToAll("StatusServerSIP", ref objParamList);
        }

        return m_bStatusRun;
  }

 public void StopServerSIP()
 {
        if(m_bStatusRun == true)
        {
            UnInitVaxTeleSIP();
            m_bStatusRun = false;
        }
     
        ArrayList objParamList = new ArrayList();
        objParamList.Add("SIP Server is not running.");

        cTelnetServer.m_objTelnetServer.SendCommandToAll("StatusServerSIP", ref objParamList);

 }

    private void UnInitVaxTeleSIP()
    {
        LogStatus(" ");

        m_objCallList.DeleteAll();
        m_objServerData.UnLoadServer();

        LogStatus("Uninitializing VaxTele SIP Server.");
        m_objVaxTeleCOM.UnInitialize();
        LogStatus("VaxTele SIP Server uninitialized successfully.");

        m_objUserList.DeleteAll();
        LogStatus("VaxTele SIP Server stopped successfully.");
    }

    private Boolean InitVaxTeleSIP()
    {
        LogStatus(" ");

        cRegistry objReg = new cRegistry();
        
        if(objReg.GetStartServer() == false)
            return false;
        
        string sListenIP = "";
        string sDomainRealm = "";
        string sListenPort = "";

        objReg.GetComm(ref sListenIP, ref sListenPort, ref sDomainRealm);

        m_objClientRegEvent.SetDomainRealm(sDomainRealm);
        m_objVaxTeleCOM.SetLicenseKey("TRIAL-LICENSE-KEY");

        LogStatus("Initializing VaxTele SIP Server.");

        if(m_objVaxTeleCOM.Initialize(sDomainRealm, sListenIP, Convert.ToInt32(sListenPort), sListenIP, -1) == false)
        {
            LogStatus("Unable to initialize VaxTele SIP Server (see error log).");
            LogError("");

            return false;
        }

        LogStatus("VaxTele SIP Server initialized successfully.");
        m_objServerData.LoadServer();
        LogStatus("VaxTele SIP Server started successfully.");

        return true;
    }

    public void BindTwoCall(ref cCallData objCallDataA, ref cCallData objCallDataB)
    {
        objCallDataA.m_objBindCallData = objCallDataB;
        objCallDataB.m_objBindCallData = objCallDataA;
    }

    public Boolean SendResponse(ref cCallData objCallData, int nStatusCode, string sReasonPhrase)
    {
        if(objCallData.m_bConnected == true)
            return false;

        return m_objVaxTeleCOM.SendResponse(objCallData.m_nCallId, nStatusCode, sReasonPhrase, "");
    }

    public void CloseCall(ref cCallData objCallData)
    {
        switch(objCallData.m_nCallType)
	    {
		    case CALL_TYPE_QUEUE:
			    m_objQueueList.RemoveQueueCall(ref objCallData);
			    break;
		
		    case CALL_TYPE_STEALTH:
			    m_objStealthList.RemoveStealthCall(ref objCallData);
			    break;
	    }

	    PlayWaveStopToCall(ref objCallData);
	    m_objStealthList.FreeCaptureCall(ref objCallData);
		
	    CloseSession(ref objCallData);
	    m_objVaxTeleCOM.CloseCall(objCallData.m_nCallId);
			
	    cUserData objUserData = m_objUserList.FindUser(objCallData.m_unUserId);
	    if(objUserData != null) objUserData.m_nCallCount--;
	
	    m_objCallList.RemoveCall(ref objCallData);
    }

    public cCallData DialCallToLine(ref cCallData objFromCallData, string sDialNo, string sLineName)
    {
        cLineData objLineData = m_objLineList.FindLine(sLineName);

        if(objLineData == null)
        {
            SendResponse(ref objFromCallData, 404, "User Not Found");
            CloseCall(ref objFromCallData);

            return null;
        }

        int nDialCall = m_objVaxTeleCOM.DialCallToLine(objLineData.m_nLineId, sDialNo, objLineData.m_sCodecList, 120);
        
        if(nDialCall == -1)
        {
            SendResponse(ref objFromCallData, 404, "User Not Found");
            CloseCall(ref objFromCallData);

            return null;
        }

        cCallData objDialCallData = m_objCallList.AddCall(nDialCall);
        BindTwoCall(ref objFromCallData,ref objDialCallData);

        return objDialCallData;
    }

    public cCallData DialCallToUser(ref cCallData objFromCallData, string sToUserId)
    {
        cUserData objUserData = m_objUserList.FindUser(sToUserId);

        if(objUserData == null)
        {
            SendResponse(ref objFromCallData, 404, "User Not Found");
            CloseCall(ref objFromCallData);

            return null;
        }

        int nDialCall = m_objVaxTeleCOM.DialCallToUser(objFromCallData.m_sCallerId, sToUserId, objUserData.m_sCodecList, 120);
        
        if(nDialCall == -1)
        {
            SendResponse(ref objFromCallData, 404, "User Not Found");
            CloseCall(ref objFromCallData);

            return null;
        }

        objUserData.m_nCallCount++;

        cCallData objDialCallData = m_objCallList.AddCall(nDialCall);

        objDialCallData.m_sUserId = sToUserId;
        objDialCallData.m_unUserId = objUserData.m_unUserId;

        BindTwoCall(ref objFromCallData,ref objDialCallData);

        return objDialCallData;
    }

    public Boolean AcceptCall(ref cCallData objCallData)
    {
        Boolean bResult = m_objVaxTeleCOM.AcceptCall(objCallData.m_nCallId, objCallData.m_sCodec, 120);

        if (bResult == false)
        {
            LogError("");
            LogStatus("cServerSIP.AcceptCall() method fails.");
        }

        return bResult;
    }

    public void CreateSession(ref cCallData objCallData)
    {
        CloseSession(ref objCallData);

        objCallData.m_nSessionId = m_objVaxTeleCOM.CreateSession();
        m_objVaxTeleCOM.AddCallToSession(objCallData.m_nCallId, objCallData.m_nSessionId);
    }

    public void AddCallToSession(ref cCallData objCallData, int nSessionId)
    {
        m_objVaxTeleCOM.AddCallToSession(objCallData.m_nCallId, nSessionId);
    }

    public void CloseSession(ref cCallData objCallData)
    {
        if(objCallData.m_nSessionId == -1)
            return;
        
        PlayWaveStopToCall(ref objCallData);

        m_objVaxTeleCOM.CloseSession(objCallData.m_nSessionId);
        objCallData.m_nSessionId = -1;
    }

    public void TransferCallAccept(ref cCallData objCallData)
    {
        m_objVaxTeleCOM.TransferCallAccept(objCallData.m_nCallId);
    }

    public void DisableDetectDTMF(ref cCallData objCallData)
    {
        objCallData.m_bDigitDetect = false;
        objCallData.m_nDigitPRESS = 0;
        objCallData.m_sDigitPRESS = "";

        m_objVaxTeleCOM.DetectDTMF(0, objCallData.m_nSessionId, false); // Disable RFC2833 DTMF detection
    }

    public void EnableDetectDTMF(ref cCallData objCallData)
    {
        objCallData.m_bDigitDetect = true;
        objCallData.m_nDigitPRESS = 0;
        objCallData.m_sDigitPRESS = "";

        if (!m_objVaxTeleCOM.DetectDTMF(0, objCallData.m_nSessionId, true)) //Enable RFC2833 DTMF detection
            LogError("");
    }

    public Boolean RejectClientRegister(int nRegId)
    {
        return m_objVaxTeleCOM.RejectClientRegister(nRegId);
    }

    public Boolean AuthClientRegister(int nRegId)
    {
        return m_objVaxTeleCOM.AuthClientRegister(nRegId);
    }

    public void OnClientUnRegister(string sUserLogin)
    {
        m_objClientRegEvent.OnClientUnRegister(sUserLogin);
    }

    public void OnClientRegister(string sUserLogin, string sDomain, int nRegId)
    {
        m_objClientRegEvent.OnClientRegister(sUserLogin, sDomain, nRegId);
    }

    public void OnClientRegisterSuccess(string sUserLogin)
    {
        m_objClientRegEvent.OnClientRegisterSuccess(sUserLogin);
    }

    public void OnClientRegisterFail(string sUserLogin)
    {
        m_objClientRegEvent.OnClientRegisterFail(sUserLogin);
    }

    public void OnIncomingCallAnonymous(int nCallId, string sCallFrom, string sCallTo, string sFromURI, string sToURI, string sFromIP, int nFromPort)
    {
        m_objVaxTeleCOM.SendResponse(nCallId, 404, "Anonymous not allowed.", "");
    }

    public void OnIncomingCallFromLine(int nCallId, int nLineId, string sCallFrom, string sCallTo, string sFromURI, string sToURI, string sFromIP, int nFromPort)
    {
        cLineData objLineData = m_objLineList.FindLine(nLineId);

        if(objLineData == null)
        {
            m_objVaxTeleCOM.CloseCall(nCallId);
            return;
        }

        cCallData objCallData = m_objCallList.AddCall(nCallId);

        objCallData.m_sCodec = objLineData.m_sCodecList;
        objCallData.m_sCallerId = sCallFrom;

        CreateSession(ref objCallData);

        m_objCallEvent.OnIncomingCallFromLine(ref objCallData, ref objLineData, sCallTo);
    }

    public void OnIncomingCallFromUser(int nCallId, string sUserName, string sCallFrom, string sCallTo, string sFromURI, string sToURI, string sFromIP, int nFromPort)
    {
        cUserData objUserData = m_objUserList.FindUser(sUserName);

        if(objUserData == null)
            return;
        
        cCallData objCallData = m_objCallList.AddCall(nCallId);

        objCallData.m_sCodec = objUserData.m_sCodecList;
        objCallData.m_unUserId = objUserData.m_unUserId;
        objCallData.m_sUserId = sUserName;
        objCallData.m_sCallerId = sCallFrom;

        CreateSession(ref objCallData);

        m_objVaxTeleCOM.EnableDialTone(nCallId, m_objServerData.GetDialToneSessionId());
        m_objCallEvent.OnIncomingCallFromUser(ref objCallData, objUserData, sCallTo);
    }

    public void StartVoiceStream(ref cCallData objCallDataA, ref cCallData objCallDataB)
    {
        CloseSession(ref objCallDataA);
        CloseSession(ref objCallDataB);

        int nSessionId = m_objVaxTeleCOM.CreateSession();

        objCallDataA.m_nSessionId = nSessionId;
        objCallDataB.m_nSessionId = nSessionId;

        AddCallToSession(ref objCallDataA, nSessionId);
        AddCallToSession(ref objCallDataB, nSessionId);

    }

    public void OnAcceptIncomingCallConnected(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objCallEvent.OnAcceptIncomingCallConnected(ref objCallData);
    }

    public void OnDialCallConnected(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objCallEvent.OnDialCallConnected(ref objCallData);
    }

    public void OnDialCallTimeOut(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objCallEvent.OnDialCallTimeOut(ref objCallData);
    }

    public void OnAcceptCallTimeOut(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objCallEvent.OnAcceptCallTimeOut(ref objCallData);
    }

    public void OnDisconnectCall(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objCallEvent.OnDisconnectCall(ref objCallData);
    }

    public void OnIncomingCallCancel(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objCallEvent.OnIncomingCallCancel(ref objCallData);
    }

    public void OnSessionLost(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;

        m_objCallEvent.OnSessionLost(ref objCallData);
    }

    public void OnProvisionalResponse(int nCallId, int nStatusCode, string sReasonPhrase)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objResponseEvent.OnProvisionalResponse(ref objCallData, nStatusCode, sReasonPhrase);
    }

    public void OnFailureResponse(int nCallId, int nStatusCode, string sReasonPhrase)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objResponseEvent.OnFailureResponse(ref objCallData, nStatusCode, sReasonPhrase);
    }

    public void OnRedirectionResponse(int nCallId, int nStatusCode, string sReasonPhrase, string sContact)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objResponseEvent.OnRedirectionResponse(ref objCallData, nStatusCode, sReasonPhrase, sContact);
    }

    public void OnDigitDetectDTMF(int nSessionId, int nCallId, int nDigit)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);
        
        if(objCallData == null)
            return;
        
        string sDigit = "0123456789*#ABCDEF";

        m_objQueueList.OnPressDigitDTMF(ref objCallData, Convert.ToString(sDigit[nDigit]));
    }

    public void OnHoldCall(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);;
        
        if(objCallData == null)
            return;
        
        m_objHoldCallEvent.OnHoldCall(ref objCallData);

    }

    public void OffHoldCall(int nCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);
        
        if(objCallData == null)
            return;
        
        m_objHoldCallEvent.OffHoldCall(ref objCallData);
    }

    public void OnTransferCallBlind(int nCallId, string sTransferee, string sTransferTo, string sTransferToURI)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);
        
        if(objCallData == null)
            return;

        m_objTransferCallEvent.OnTransferCallBlind(ref objCallData, sTransferee, sTransferTo, sTransferToURI);
    }

    public void OnTransferCallAttend(int nCallId, int nTransferToCallId)
    {
        cCallData objCallData = m_objCallList.FindCall(nCallId);
        
        if(objCallData == null)
            return;
        
        cCallData objTransferToCallData = m_objCallList.FindCall(nTransferToCallId);
        
        if(objTransferToCallData == null)
            return;
        
        m_objTransferCallEvent.OnTransferCallAttend(ref objCallData, ref objTransferToCallData);
    }

    public Boolean PlayWaveStopToCall(ref cCallData objCallData)
    {
        if(objCallData.m_nPlayWaveType == PLAY_WAVE_NONE_TYPE)
            return true;
        
        if(objCallData.m_nPlayWaveType == PLAY_WAVE_DONE_TYPE)
            m_objPlayWaveList.Remove(objCallData.m_nSessionId);
        
        objCallData.m_nPlayWaveType = PLAY_WAVE_NONE_TYPE;

        return m_objVaxTeleCOM.PlayWaveStopToSession(objCallData.m_nSessionId);
    }

    public Boolean PlayWaveStartToCall(ref cCallData objCallData, int nPlayWaveId, Boolean bLoopPlay)
    {
        if(bLoopPlay == false)
        {
            objCallData.m_nPlayWaveType = PLAY_WAVE_DONE_TYPE;
            m_objPlayWaveList.Add(objCallData.m_nSessionId, objCallData);
        }
        else
        {
            objCallData.m_nPlayWaveType = PLAY_WAVE_LOOP_TYPE;
        }

        return m_objVaxTeleCOM.PlayWaveStartToSession(nPlayWaveId, objCallData.m_nSessionId, bLoopPlay);
    }

    public void OnPlayWaveToSessionDone(int nPlayWaveId, int nSessionId)
    {
        cCallData objCallData = (cCallData) m_objPlayWaveList[nSessionId];

        if(objCallData == null)
            return;
        
        m_objPlayWaveList.Remove(nSessionId);
        objCallData.m_nPlayWaveType = PLAY_WAVE_NONE_TYPE;

        m_objQueueList.OnPlayWaveDoneToCall(ref objCallData);
    }

    public void OnLineRegisterTrying(int nLineId)
    {
        m_objLineRegisterEvent.OnLineRegisterTrying(nLineId);
    }

    public void OnLineRegisterFail(int nLineId, int nStatusCode, string sReasonPhrase)
    {
        m_objLineRegisterEvent.OnLineRegisterFail(nLineId, nStatusCode, sReasonPhrase);
    }

    public void OnLineRegisterSuccess(int nLineId)
    {
        m_objLineRegisterEvent.OnLineRegisterSuccess(nLineId);
    }

    public void OnLineUnRegisterTrying(int nLineId)
    {
        m_objLineRegisterEvent.OnLineUnRegisterTrying(nLineId);
    }

    public void OnLineUnRegisterFail(int nLineId, int nStatusCode, string sReasonPhrase)
    {
        m_objLineRegisterEvent.OnLineUnRegisterFail(nLineId, nStatusCode, sReasonPhrase);
    }

    public void OnLineUnRegisterSuccess(int nLineId)
    {
        m_objLineRegisterEvent.OnLineUnRegisterSuccess(nLineId);
    }

    public void OnVaxTeleTick(int nTickId)
    {
        if(nTickId == QUEUE_TICK_ID)
            m_objQueueList.ProcessQueueTick();
    }

    public void LogError(string sError)
    {
        m_objErrorLog.LogError(sError);
    }

    public void LogStatus(string sStatus)
    {
        ArrayList objParamList = new ArrayList();
        objParamList.Add(sStatus);

        cTelnetServer.m_objTelnetServer.SendCommandToAll("StatusLog", ref objParamList);
    }

    public Boolean LoadWaveFileToSession(string sFileName, ref int nPlayWaveId, ref int nSessionId)
    {
        LogStatus("Loading [ " + sFileName + " ] wave file....");

        int nPlayWaveLoad = m_objVaxTeleCOM.PlayWaveLoadFile(sFileName);
        
        if(nPlayWaveLoad == -1)
        {
            LogError("");
            LogStatus("Loading [ " + sFileName + " ] wave file fail.");

            return false;
        }

        int nSessionPlayWave = m_objVaxTeleCOM.CreateSession();
        
        if(nSessionPlayWave == -1)
        {
            LogError("");
            LogStatus("Loading [ " + sFileName + " ] wave file fail.");

            m_objVaxTeleCOM.PlayWaveUnLoadFile(nPlayWaveLoad);

            return false;
        }

        if(m_objVaxTeleCOM.PlayWaveStartToSession(nPlayWaveLoad, nSessionPlayWave, true) == false)
        {
            LogError("");
            LogStatus("Loading [ " + sFileName + " ] wave file fail.");

            m_objVaxTeleCOM.CloseSession(nSessionPlayWave);
            m_objVaxTeleCOM.PlayWaveUnLoadFile(nPlayWaveLoad);

            return false;
        }

        nPlayWaveId = nPlayWaveLoad;
        nSessionId = nSessionPlayWave;

        LogStatus("Loading [ " + sFileName + " ] wave file done.");
        return true;

    }

    public void UnLoadWaveFileToSession(int nPlayWaveId, int nSessionId)
    {
        m_objVaxTeleCOM.CloseSession(nSessionId);
        m_objVaxTeleCOM.PlayWaveUnLoadFile(nPlayWaveId);
    }

    public Boolean AddLine(int nLineId, string sUserName, string sLogin, string sLoginPwd, string sDisplayName, string sDomainRealm, string sProxyURI, string sOutboundProxy)
    {
        return m_objVaxTeleCOM.AddLine(nLineId, sUserName, sLogin, sLoginPwd, sDisplayName, sDomainRealm, sProxyURI, sOutboundProxy);
    }

    public Boolean RegisterLine(int nLineId)
    {
        return m_objVaxTeleCOM.RegisterLine(nLineId);
    }

    public Boolean UnRegisterLine(int nLineId)
    {
        return m_objVaxTeleCOM.UnRegisterLine(nLineId);
    }

    public int PlayWaveLoadFile(string sFileName)
    {
        return m_objVaxTeleCOM.PlayWaveLoadFile(sFileName);
    }

    public void PlayWaveUnLoadFile(int nPlayWaveId)
    {
        m_objVaxTeleCOM.PlayWaveUnLoadFile(nPlayWaveId);
    }

    public Boolean SetVaxTeleTick(int nTickId, int nElapse)
    {
        return m_objVaxTeleCOM.SetVaxTeleTick(nTickId, nElapse);
    }

    public Boolean KillVaxTeleTick(int nTickId)
    {
        return m_objVaxTeleCOM.KillVaxTeleTick(nTickId);
    }

    public Boolean AddUser(string sUserLogin, string sLoginPwd)
    {
        return m_objVaxTeleCOM.AddUser(sUserLogin, sLoginPwd);
    }

    public void RemoveUser(string sUserLogin)
    {
        m_objVaxTeleCOM.RemoveUser(sUserLogin);
    }

    public int GetVaxObjectError()
    {
        return m_objVaxTeleCOM.GetVaxObjectError();
    }

    public void OnTelnetCommand(string sCommand, ref ArrayList objParamList, ref cTelnetClient objTelnetClient)
    {
        cTelnetCmd objTelnetCmd = new cTelnetCmd();

        if(String.Compare(sCommand, "ShowUsers", true) == 0)
        {
            objTelnetCmd.ShowUsers(ref objTelnetClient);
            return;
        }
    

        if(String.Compare(sCommand, "UserInfo", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.UserInfo(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "AddUser", true) == 0 && objParamList.Count == 5)
        {
            objTelnetCmd.AddUser(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "RemoveUser", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.RemoveUser(ref objParamList, ref objTelnetClient);
            return;
        }
    
        if(String.Compare(sCommand, "ShowQueues", true) == 0)
        {
            objTelnetCmd.ShowQueues(ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "QueueInfo", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.QueueInfo(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "AddQueue", true) == 0 && objParamList.Count == 6)
        {
            objTelnetCmd.AddQueue(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "RemoveQueue", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.RemoveQueue(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "ShowLines", true) == 0)
        {
            objTelnetCmd.ShowLines(ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "LineInfo", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.LineInfo(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "AddLine", true) == 0 && objParamList.Count == 12)
        {
            objTelnetCmd.AddLine(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "RemoveLine", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.RemoveLine(ref objParamList, ref objTelnetClient);
            return;
        }

        ///////////////////////////

        if(String.Compare(sCommand, "ShowDialPlans", true) == 0)
        {
            objTelnetCmd.ShowDialPlans(ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "DialPlanInfo", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.DialPlanInfo(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "AddDialPlan", true) == 0 && objParamList.Count == 3)
        {
            objTelnetCmd.AddDialPlan(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "RemoveDialPlan", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.RemoveDialPlan(ref objParamList, ref objTelnetClient);
            return;
        }

        ///////////////////////////

        if(String.Compare(sCommand, "SetStealthMusic", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.SetStealthMusic(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "GetStealthMusic", true) == 0)
        {
            objTelnetCmd.GetStealthMusic(ref objTelnetClient);
            return;
        }

        //////////////////////////

        if(String.Compare(sCommand, "SetHoldMusic", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.SetHoldMusic(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "GetHoldMusic", true) == 0)
        {
            objTelnetCmd.GetHoldMusic(ref objTelnetClient);
            return;
        }

        //////////////////////////

        if(String.Compare(sCommand, "SetDialToneMusic", true) == 0 && objParamList.Count == 1)
        {
            objTelnetCmd.SetDialToneMusic(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "GetDialToneMusic", true) == 0)
        {
            objTelnetCmd.GetDialToneMusic(ref objTelnetClient);
            return;
        }

        //////////////////////////

        if(String.Compare(sCommand, "SetCOMM", true) == 0 && objParamList.Count == 3)
        {
            objTelnetCmd.SetCOMM(ref objParamList, ref objTelnetClient);
            return;
        }

        if(String.Compare(sCommand, "GetCOMM", true) == 0)
        {
            objTelnetCmd.GetCOMM(ref objTelnetClient);
            return;
        }

        //////////////////////////

        if(String.Compare(sCommand, "ServerStart", true) == 0)
        {
            objTelnetCmd.StartServer(ref objTelnetClient);

            StartServerSIP();
            return;
        }

        if(String.Compare(sCommand, "ServerStop", true) == 0)
        {
            objTelnetCmd.StopServer(ref objTelnetClient);

            StopServerSIP();
            return;
        }

        if(String.Compare(sCommand, "StatusServerSIP", true) == 0)
        {
            ArrayList objSendParamList = new ArrayList();

            if(m_bStatusRun)
            {
                objSendParamList.Add("SIP Server is running.");
                objTelnetClient.SendCommand("StatusServerSIP", ref objSendParamList);
            }
            else
            {
                objSendParamList.Add("SIP Server is not running.");
                objTelnetClient.SendCommand("StatusServerSIP", ref objSendParamList);
            }
        }

        ////////////////////////

        if(String.Compare(sCommand, "GetCallCount", true) == 0)
        {
            ArrayList objSendParamList = new ArrayList();

            string sCallCount = Convert.ToString(m_objCallList.GetCount());
            objSendParamList.Add(sCallCount);

            objTelnetClient.SendCommand("GetCallCount", ref objSendParamList);
            return;
        }

        if(String.Compare(sCommand, "GetUserCount", true) == 0)
        {
            ArrayList objSendParamList = new ArrayList();

            string sUserCount = Convert.ToString(m_objUserList.GetCount());
            objSendParamList.Add(sUserCount);

            objTelnetClient.SendCommand("GetUserCount", ref objSendParamList);
            return;
        }

        if(String.Compare(sCommand, "GetUsageCPU", true) == 0)
        {
            ArrayList objSendParamList = new ArrayList();

            string sUsageCPU = Convert.ToString((int) m_objUsageCPU.NextValue());
            objSendParamList.Add(sUsageCPU);

            objTelnetClient.SendCommand("GetUsageCPU", ref objSendParamList);
            return;
        }

        if (String.Compare(sCommand, "GetAvailMEM", true) == 0)
        {
            ArrayList objSendParamList = new ArrayList();
            
            PerformanceCounter objUsageMEM = new PerformanceCounter("Memory", "Available KBytes"); 
            string sAvailMEM = Convert.ToString(objUsageMEM.NextValue());

            objSendParamList.Add(sAvailMEM);

            objTelnetClient.SendCommand("GetAvailMEM", ref objSendParamList);
            return;
        }

        ////////////////////////

        objTelnetCmd.InvalidCommand(ref objTelnetClient);

    }

}
